/**
 * 
 */
package org.gcube.accounting.datamodel.usagerecords;

import java.util.HashSet;
import java.util.Set;

import org.gcube.accounting.datamodel.UsageRecord;
import org.gcube.accounting.datamodel.basetypes.AbstractTaskUsageRecord;
import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
import org.gcube.documentstore.exception.InvalidValueException;
import org.gcube.documentstore.records.Record;
import org.gcube.testutility.ScopedTest;
import org.gcube.testutility.TestUsageRecord;
import org.gcube.testutility.TestUtility;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR) http://www.lucafrosini.com/
 *
 */
public class TaskUsageRecordScopedTest extends ScopedTest {

	private static Logger logger = LoggerFactory.getLogger(TaskUsageRecordScopedTest.class);
	
	@Before
	public void before() throws Exception{
		SecurityTokenProvider.instance.set(TestUtility.TOKEN);
	}
	
	@After
	public void after() throws Exception{
		SecurityTokenProvider.instance.reset();
	}
	
	public static Set<String> getExpectedRequiredFields(){
		Set<String> expectedRequiredFields = new HashSet<String>();
		expectedRequiredFields.add(Record.ID);
		expectedRequiredFields.add(UsageRecord.CONSUMER_ID);
		expectedRequiredFields.add(UsageRecord.CREATION_TIME);
		expectedRequiredFields.add(UsageRecord.RECORD_TYPE);
		expectedRequiredFields.add(UsageRecord.SCOPE);
		expectedRequiredFields.add(UsageRecord.OPERATION_RESULT);
		expectedRequiredFields.add(AbstractTaskUsageRecord.TASK_ID);
		expectedRequiredFields.add(AbstractTaskUsageRecord.TASK_START_TIME);
		expectedRequiredFields.add(AbstractTaskUsageRecord.TASK_END_TIME);
		
		return expectedRequiredFields;
	}
	
	@Test
	public void testRequiredFieldsScoped() throws InvalidValueException{
		TaskUsageRecord usageRecord = TestUsageRecord.createTestTaskUsageRecord();
		
		Set<String> expectedRequiredFields = TaskUsageRecordTest.getExpectedRequiredFields();
		logger.debug("Expected Required Fields : {}", expectedRequiredFields);
		
		Set<String> gotRequiredFields = usageRecord.getRequiredFields();
		logger.debug("Got Required Fields : {}", gotRequiredFields);
		
		Assert.assertTrue(expectedRequiredFields.containsAll(gotRequiredFields));
		Assert.assertTrue(gotRequiredFields.containsAll(expectedRequiredFields));
		
		usageRecord.validate();
		logger.debug("{}", usageRecord);
	}

}
